#include "stdio.h"
#include "fcntl.h"
#include "termp.h"
/*
#include "srvconst.h"
*/

/*
-- ----------------------------------------------------------------------------
-- Termp External Variables.
-- ----------------------------------------------------------------------------
*/

extern int last_screen_line, last_screen_col ;

/*
-- ----------------------------------------------------------------------------
-- Termp Static Variables.
-- ----------------------------------------------------------------------------
*/

static int Cx, Cy;
/*static int WellDone = 1;*/

void abort ()
/*
  Creation of the empty file "@abort" signals an abnormal end
  (useful to test for the existence of this file in scripts).
*/
{
  open ("@abort", (O_CREAT | O_TRUNC) | O_WRONLY, 511) ;
  exit (1) ;
}

/*
-- ----------------------------------------------------------------------------
-- void TrackCursor - Update screen.p 's idea of where the cursor is.
--
-- Input Parameters:
--   (int) Ch - Current value of cursor.
--
-- Output Parameters:
--   None.
--
-- NOTES:
-- This will be the same as the actual cursor position unless we recieve
-- characters which are not normal visible characters ( ' ' - '~', cr, lf).
-- ----------------------------------------------------------------------------
*/
void TrackCursor (Ch)
int Ch;
{
  switch (Ch)
    {
      case '\r':
        {
          Cx = 0;
          break;
        }
      case '\n':
        {
          if (Cy != last_screen_line)
            Cy++;
          break;
        }
      case '\b':
        {
          if (Cx != FIRST_SCREEN_COL)
            Cx--;
          break;
        }
      default:
        {
          Cx++;
          break;
        }
    };
}

int do_screen (buffer, in_ptr)
char *buffer ;
int  *in_ptr ;

/* 
'buffer' is the main buffer. On entry it contains the header just
read from the link.  'in_ptr' is an (int) pointer
into this buffer, pointing to the next word to be read from
the header.

A negative return value indicates an irrecoverable error.  Otherwise
return value is 0.

Most of the TDS 'to.term.p' protocol is implemented.  Various TDS
specific capabilities, such as 'tt.terminate', 'tt.help', 'tt.initialise',
'tt.key.raw' and 'tt.key.cooked' are not supported.
*/

{
  int Command;
  Command = *(in_ptr++) ;

  switch (Command)
  {
    case  TT_RESET:
    {
      Cx = 0; Cy = 0;
      GotoXY (Cx, Cy);
      break;
    }
    case  TT_ABORT:
    {
      abort () ;
      break;
    }
    case  TT_UP:
    {
      if (Cy)
        Cy = Cy - 1;
      GotoXY (Cx, Cy);
      break;
    }
    case  TT_DOWN:
    {
      Cy = Cy + 1;
      GotoXY (Cx, Cy);
      break;
    }
    case  TT_LEFT:
    {
      if (Cx)
        Cx = Cx - 1;
      GotoXY (Cx, Cy);
        break;
    }
    case  TT_RIGHT:
    {
      Cx = Cx + 1;
      GotoXY (Cx, Cy);
      break;
    }
    case  TT_CLEAR_EOS:
    {
      ClearEos ();
      break;
    }
    case  TT_CLEAR_EOL:
    {
      ClearEol () ;
      break;
    }
    case  TT_INS_LINE:
    {
      if (! InsLine (Cx, Cy))
        RingBell () ;
      break;
    }
    case  TT_DEL_LINE:
    {
      if (! DelLine (Cx, Cy))
        RingBell () ;
      break;
    }
    case  TT_GOTO:
    {
      Cx = *(in_ptr++) ;
      Cy = *(in_ptr++) ;
      GotoXY (Cx, Cy);
      break;
    }
    case  TT_OUT_STRING:
    {
      int Length = *(in_ptr++) ;
      char *ScreenPBuffer = (char *) in_ptr ;
      ScreenPBuffer[Length] = '\0';           /* round off our string nicely */
      if (Length>0)
      {
        Length = 0;
        while (ScreenPBuffer[Length] != '\0')
          TrackCursor (ScreenPBuffer[Length++]);
        Print (ScreenPBuffer) ;
      }
      break;
    }
    case  TT_OUT_BYTE:
    {
      int Ch = *(in_ptr++) ;
      TrackCursor (Ch);
      PutChar (Ch) ;
      break;
    }
    case  TT_OUT_INT:
    {
      int Ch = *(in_ptr++) ;
      TrackCursor (Ch);
      PutChar (Ch) ;
      break;
    }
    case  TT_TERMINATE:
    case  TT_INS_CHAR:
    case  TT_DEL_CHAR:
    case  TT_BEEP:
    case  TT_HELP:
    case  TT_INITIALISE:
    case  TT_KEY_RAW:
    case  TT_KEY_COOKED:
    {
      RingBell () ;
      break;
    }
    default:
    {
      printf ("Unknown term-p tag: %d\n", Command) ;
      return (-1) ;
    }
  }
  return (0) ;
}
/*
--
--     End of File.
--
*/
